# Copyright (c) 2007 Michael "Z" Goddard
# This software is distributed under the MIT license.

"""
All the classes and functions that respond to the forms of the templates.

These response classes follow a common theme of having methods that share 
there name with the names of the submit input on there corresponding 
templates.  Using these names as hooks those methods go about the business of
performing what action it should.  Or at least this is what they will end up
like.
"""

import os
import time
import shutil
import mimetypes
import types
import sys

from Pyblosxom.pyblosxom import Request, \
     blosxom_process_path_info, \
     blosxom_entry_parser, \
     blosxom_file_list_handler
from Pyblosxom import tools

from template import *
from webfront import *
from fileops import *

try:
    import Cheetah.Template
    import Cheetah.ImportHooks
except:
    # um, they are most likely missing, since they cannot be loaded the 
    # webfront cannot display any templates.  This is rather unfortunate ...
    #
    # For the time being reraise it, but a better solution needs to be devised
    # later.
    raise

def new_handle(webfront):
    webfront.action = 'edit'
    webfront.editpage()

def preview_handle(webfront):
    webfront.request.data['extensions'] = {'draft': blosxom_entry_parser}
    pyhttp = webfront.request.http
    pyhttp['PATH_INFO'] = pyhttp['PATH_INFO'][:pyhttp['PATH_INFO'].rfind(os.extsep)+1] + "html"
    tools.run_callback("pathinfo",
        {"request": webfront.request},
        defaultfunc=blosxom_process_path_info)
    webfront.handled = 0
    webfront.toRender = 0

def file_handle(webfront):
    webfront.toRender = 0
    if file_punter(webfront.basedir, webfront.request):
        webfront.handled = 1
    else:
        webfront.handled = 0

class EntryPage(Template):
    def __init__(self, webfront):
        super(EntryPage, self).__init__(webfront, "entry")
    
    def __call__(self, webfront, runpathinfo=False):
        self.runpathinfo = runpathinfo
        super(EntryPage, self).__call__(webfront)
    
    # template variable methods
    def directories(self):
        self.checkPathInfo()
        
        dirstr = ""
        # directory view
        directories = []
        if self.webfront.mode == 'post':
            directories = webfront_directory_scan(self.request.data['root_datadir'])
            self.request.data['directoryaction'] = "edit"
            self.webfront.render("title_header")
        else:
            directories = webfront_directory_scan(self.webfront.pathdir)
            self.request.data['directoryaction'] = "file"
        if self.request.data['pi_bl'].count(os.sep) > 1 or len(self.request.data['pi_bl'][self.request.data['pi_bl'].rfind(os.sep):]) > 1 or\
            self.request.http['PATH_INFO'].count(os.sep) > 1 or len(self.request.http['PATH_INFO'][self.request.http['PATH_INFO'].rfind(os.sep):]) > 1:
            self.request.data["directory"] = "[Up a directory]"
            self.request.data["directorypath"] = self.url[:self.url.rfind(os.sep, 0, self.url.rfind('?'))+1] + "?edit=entry&mode=" + self.webfront.mode
            self.request.data["contains"] = "Go to the parent directory"
            dirstr += self.sub['directories']()
        for directory in directories:
            if directory == '.':
                continue
            self.request.data["directory"] = directory
            self.request.data["directorypath"] = os.path.join(self.url, directory) + os.sep + "?edit=entry&mode=" + self.webfront.mode
            self.request.data["contains"] = "%d files and %d directories" % (len(directories[directory]["files"]), len(directories[directory]["directories"]))
            self.request.data['movepath'] = os.path.join(self.url, directory) + "?edit=move&mode=" + self.webfront.mode
            self.request.data["deletepath"] = os.path.join(self.url, directory) + "?edit=delete&mode=" + self.webfront.mode
            self.request.data['directory_delete'] = self.sub['directory_delete'](self.request)
            dirstr += self.sub['directories'](self.request)
            self.webfront.flash(repr(self.sub['directory_checkbox']))
        return dirstr
    
    def stories(self):
        self.checkPathInfo()
        webfront = self.webfront
        data = self.request.data
        http = self.request.http
        conf = self.request.conf
        
        self.files = []
        if self.webfront.mode == 'post':
            if not os.access(self.webfront.config['datadir'] + self.webfront.http['PATH_INFO'], os.F_OK):
                self.webfront.error("""Hmm, seems that %s does not exist, you may want to
                go to <a href='%s?edit=edit'>here</a>.""" % (self.webfront.http['PATH_INFO'], self.webfront.config['base_url']))
            self.files = tools.run_callback("filelist",
                {"request": self.request},
                defaultfunc=blosxom_file_list_handler)
        else:
            # path_info should point to the folder on top of self.webfront.basedir, so,
            # go ahead and pull a list of files
            self.files = webfront_file_list(self.webfront.pathdir, self.webfront.extfilter)
            
        stories = ""
        # entries / files
        for entry in self.files:
            if self.webfront.mode == 'post':
                data["file_name"] = entry.getId().split(os.sep)[-1]
                data["editpath"] = self.request.config["base_url"] + entry.getId()[len(self.request.config["datadir"]):] + "?edit=edit"
                data["title"] = entry.getMetadata("title", "no title")
                data['title_cell'] = self.title_cell()
                data["viewpath"] = data["editpath"][:data["editpath"].rfind(os.extsep)] + ".html"
                if data['file_name'][data['file_name'].rfind(os.extsep)+1:] == "draft":
                    data['viewpath'] += "?edit=preview"
                data["movepath"] = conf["base_url"] + entry.getId()[len(self.request.config['datadir']):] + "?edit=move"
                data["deletepath"] = self.request.config["base_url"] + entry.getId()[len(self.request.config['datadir']):] + "?edit=delete"
            else:
                data['file_name'] = entry['name']
                # do we need to show the editpath
                self.entry = entry
                type, encoding = mimetypes.guess_type(entry['name'])
                if type == 'text/plain' or entry['ext'] in self.webfront.textextensions:
                    data['editpath'] = self.request.config['base_url'] + os.sep + self.request.http['PATH_INFO'].lstrip(os.sep) + entry['name'] + "?edit=edit&mode=" + self.webfront.mode
                data['viewpath'] = self.url + entry['name'] + "?edit=file&mode=" + self.webfront.mode
                data['movepath'] = self.url + entry['name'] + "?edit=move&mode=" + self.webfront.mode
                data['deletepath'] = self.url + entry['name'] + "?edit=delete&mode=" + self.webfront.mode
            data["date"] = time.strftime("%D", time.gmtime(entry.get("mtime", 0.0)))
            stories += self.sub['stories'](self.request)
        return stories
    
    def editlink(self):
        if self.webfront.mode == 'post':
            return self.sub['editlink']()
            
        elif self.entry['ext'] in self.webfront.textextensions:
            return self.sub['editlink']()
        return ""
    
    def newPostTab(self):
        if self.webfront.mode == "post":
            return self.linktab_bot("?edit=new&mode=post", "New Post")
        return ""
    
    def storyrow(self):
        if not hasattr(self, "row"):
            self.row = 0
        self.row += 1
        return self.row % 2
    
    # form submit response method
    def submit_entry_move(self):
        # moving multiple files/directories
        self.webfront.movepage()
    
    def submit_entry_delete(self):
        # deleting multiple files/directories
        self.webfront.deletepage()
    
    # form response associated methods
    def checkPathInfo(self):
        # PATH_INFO work to make sure to display a folder
        if os.path.isdir(self.webfront.basedir + self.webfront.http['PATH_INFO']):
            # well, we want it to be a directory so maintain course
            pass
        elif os.access(self.webfront.basedir + self.webfront.http['PATH_INFO'], os.F_OK):
            # hmm, it doesn't exist, well to the directory then
            self.webfront.http['PATH_INFO'] = self.webfront.http['PATH_INFO'].rstrip(os.sep)
            self.webfront.http['PATH_INFO'] = self.webfront.http['PATH_INFO'][:self.webfront.http['PATH_INFO'].rfind(os.sep)+1]
            self.runpathinfo = True
        else:
            # hmm, its a file or it doesn't exist
            self.webfront.http['PATH_INFO'] = self.webfront.http['PATH_INFO'][:self.webfront.http['PATH_INFO'].rfind(os.sep)+1]
            self.runpathinfo = True
            
        self.webfront.data["path"] = self.webfront.http['PATH_INFO']
        pathdir = os.path.join(self.webfront.basedir, self.webfront.http['PATH_INFO'].strip(os.sep))
        
        if not self.webfront.data.has_key('bl_type') or self.runpathinfo:
            # path_info can point to something as pyblosxom does
            tools.run_callback("pathinfo",
                {"request": self.request},
                defaultfunc=blosxom_process_path_info)
        
        url = self.webfront.config['base_url'] + self.webfront.http['PATH_INFO'].rstrip(os.sep) + os.sep
        self.url = url
    

class MovePage(Template):
    def __init__(self, webfront):
        super(MovePage, self).__init__(webfront, "move")
    
    # template variable methods
    def rename(self):
        return self.webfront.form.getfirst('rename', 
            self.webfront.pathdir.split(os.sep)[-1])
    
    def move_type(self):
        webfront = self.webfront
        form = self.webfront.form
        if form.has_key('file'):
            return "multiple"
        elif os.path.isdir(webfront.pathdir):
            return "directory"
        elif os.path.isfile(webfront.pathdir):
            return "file"
        return "unknown"
    
    def files(self):
        http = self.request.http
        form = self.webfront.form
        if form.has_key('file') and form.has_key('directory'):
            return form.getlist('file') + form.getlist('directory')
        if form.has_key('file'):
            return form.getlist('file')
        if form.has_key('directory'):
            return form.getlist('directory')
        if os.path.isfile(self.webfront.pathdir):
            return http['PATH_INFO'].split(os.sep)[-1]
        if os.path.isdir(self.webfront.pathdir):
            return http['PATH_INFO'].rstrip(os.sep).split(os.sep)[-1]
        return ""
    
    def form_files(self, *args):
        if len(args):
            return self.sub['form_files'](*args)
        form = self.webfront.form
        if form.has_key('file') or form.has_key('directory'):
            files = ""
            flist = []
            if form.has_key('file'):
                flist.extend(form.getlist('file'))
            if form.has_key('directory'):
                flist.extend(form.getlist('directory'))
            for f in flist:
                files += self.sub['form_files'](f)
            return files
        return ""
    
    def directory_list(self, selectname):
        webfront = self.webfront
        data = webfront.data
        form = webfront.form
        dirs = ""
        movdir = []
        if form.has_key('file'):
            for f in form.getlist('file'):
                if os.path.isdir(webfront.pathdir + os.sep + f):
                    movdir.append(webfront.http['PATH_INFO'].strip(os.sep) + os.sep + f)
        elif os.path.isdir(webfront.pathdir):
            movdir.append(webfront.http['PATH_INFO'].strip(os.sep))
        def parents(directory, dirlist):
            if not len(dirlist):
                return False
            directory = directory[dirlist[0].rfind(os.sep):]
            for d in dirlist:
                d = d[d.rfind(os.sep)]
                if directory.startswith(d):
                    return True
            return False
        
        filepath = form.getfirst(selectname, os.sep)
        while not os.path.isdir(webfront.basedir + filepath):
            filepath = filepath[:filepath.rfind(os.sep)+1]
        directories = webfront_directory_scan(webfront.basedir, 
            depthlimit=99)
        dirsort = directories.keys()
        dirsort.sort()
        choice = selected = ''
        for directory in dirsort:
            if directory in movdir or parents(directory, movdir):
                continue
            if directory == ".":
                choice = os.sep
            else:
                choice = os.sep + directory + os.sep
            if choice == filepath:
                selected = "selected"
            else:
                selected = ""
            dirs += self.select_option(choice, selected)
        return dirs
    
    # submit response method
    def submit_move(self):
        webfront = self.webfront
        data = self.request.data
        http = self.request.http
        conf = self.request.conf
        form = self.webfront.form
        
        self.cantmove = []
        
        if not form.has_key('dest'):
            webfront.error("Please select a destination.")
            webfront.result = False
            return
        
        destdirectory = webfront.basedir + form.getfirst('dest')
        
        if form.has_key('file') or form.has_key('directory'):
            # multiple operation
            for item in form.getlist('file') + form.getlist('directory'):
                path = webfront.pathdir.rstrip(os.sep) + os.sep + item
                if os.path.isfile(path):
                    self.moveFile(path, destdirectory + item)
                elif os.path.isdir(path):
                    self.moveDirectory(path, destdirectory + item)
                # we'll ignore anything else since it clearly doesn't exist,
                # additionally we may have already moved it.
        elif os.path.isfile(webfront.pathdir):
            # file operation
            item = form.getfirst('rename', webfront.pathdir.split(os.sep)[-1])
            self.moveFile(webfront.pathdir, destdirectory + item)
        elif os.path.isdir(webfront.pathdir):
            # directory operation
            item = form.getfirst('rename', webfront.pathdir.split(os.sep)[-1])
            self.moveDirectory(webfront.pathdir, destdirectory + item)
        
        if len(self.cantmove):
            webfront.replacepage(self.cantmove)
        else:
            webfront.result = False
            webfront.entrypage()
    
    def moveFile(self, src, dst):
        replacing = self.webfront.replacecheck(
                        dst,
                        self.webfront.mode == "post")
        if replacing == True or replacing == "not replacing":
            shutil.move(src, dst)
        elif isinstance(replacing, str):
            self.cantmove.append(src[src.rfind(os.sep):])
    
    def moveDirectory(self, src, dst):
        replacing = self.webfront.replacecheck(
                        dst,
                        self.webfront.mode == "post")
        if replacing == True or replacing == "not replacing":
            shutil.move(src, dst)
        elif isinstance(replacing, str):
            src = src.rstrip(os.sep)
            self.cantmove.append(src[src.rfind(os.sep):])
    

class NewdirPage(Template):
    def __init__(self, webfront):
        super(NewdirPage, self).__init__(webfront, "newdir")
    
    # template variable
    def newdir_name(self):
        return self.webfront.form.getfirst('newdir_name', "")
    
    # submit reponse
    def submit_create(self):
        webfront = self.webfront
        config = self.webfront.conf
        http = self.webfront.http
        form = self.webfront.form
        pathdir = webfront.pathdir
        
        if self.validate():
            return
        
        if os.access(pathdir + form.getfirst("newdir_name", ""), os.F_OK):
            # The directory already exists
            self.webfront.error("The directory '%s', already exists." % form.getfirst("newdir_name"))
            return
        if not os.access(pathdir, os.F_OK):
            # The parent directory doesn't exist
            self.webfront.error("Well this is odd, but the directory that you wish to create another directory in, does not exist.")
            return
        os.mkdir(pathdir + form.getfirst('newdir_name'))
        webfront.entrypage(runpathinfo=True)
    
    def validate(self):
        form = self.webfront.form
        error = self.webfront.error
        if not form.has_key('newdir_name'):
            error("You need to name the direcdtory before it can be made.")
            return True
        if not form.getfirst('newdir_name').replace(os.extsep, '').replace('_', '').replace('-', '').isalnum():
            error("""Make sure to only use letters of the alphabet, numerals,
            periods, underscores, and dashes in the name.""")
            return True
        return False
    

class UploadPage(Template):
    def __init__(self, webfront):
        super(UploadPage, self).__init__(webfront, "upload")
    
    # template variables
    def file_dest(self):
        return self.webfront.basedir
    
    def directory_list(self, selectname):
        form = self.webfront.form
        http = self.webfront.http
        dirs = ""
        filepath = form.getfirst(selectname, os.sep + http['PATH_INFO'].strip(os.sep) + os.sep)
        if not os.path.isdir(filepath):
            filepath = filepath[:filepath.rfind(os.sep)+1]
        directories = webfront_directory_scan(self.webfront.basedir, 
            depthlimit=99)
        dirsort = directories.keys()
        dirsort.sort()
        choice = selected = ''
        for directory in dirsort:
            if directory == ".":
                choice = os.sep
            else:
                choice = os.sep + directory + os.sep
            if choice == filepath:
                selected = "selected"
            else:
                selected = ""
            dirs += self.select_option(choice, selected)
        return dirs
    
    # submit response
    def submit_upload(self):
        http = self.webfront.http
        config = self.webfront.config
        if not self.webfront.form.has_key('upload_file'):
            self.webfront.error("Make sure to select a file to upload.")
            return
        fileitem = self.webfront.form['upload_file']
        
        directory = os.path.join(self.webfront.basedir, self.webfront.getfield('path', os.sep).strip(os.sep))
        # incase the path_info includes a file
        if os.path.isfile(directory):
            directory = directory[:directory.rfind(os.sep)] 
        filepath = os.path.join(directory, fileitem.filename)
        replacing = self.webfront.replacecheck(filepath, self.webfront.mode == 'post')
        if replacing == False:
            self.webfront.error("Please choose a file with a name that is already used or a different directory.")
            self.webfront.result = False
        elif replacing not in ['not replacing', True]:
            self.webfront.replacepage(os.path.join(directory, replacing), "")
            return
        if not upload_file(filepath, fileitem.file):
            self.webfront.error("Upload of %s failed." % filepath)
            
        else:
            self.webfront.flash("Upload of %s succeeded." % filepath)
            self.webfront.entrypage()
    

class ReplacePage(Template):
    def __init__(self, webfront):
        super(ReplacePage, self).__init__(webfront, "replace")
    
    def __call__(self, webfront, replacing, oldfile=None):
        self.replacing = replacing
        self.oldfile = oldfile
        super(ReplacePage, self).__call__(webfront)
    
    # template variables
    def replaced_file(self):
        return self.replacing
    
    def old_file(self):
        if self.oldfile:
            return self.oldfile
        return ""
    
    def formcopy(self, *args):
        if len(args):
            return self.sub['formcopy'](*args)
        ret = ""
        ignore = ["replace"]
        form = self.webfront.form
        data = self.request.data
        for fieldkey in form.keys():
            if fieldkey in ignore:
                continue
            for fieldvalue in form.getlist(fieldkey):
                ret += self.formcopy(fieldkey, fieldvalue)
        return ret
    

class LoginPage(Template):
    def __init__(self, webfront):
        super(LoginPage, self).__init__(webfront, "login")
    
    def initialize(self):
        test_result = test_configuration(self.webfront.conf)
        if test_result:
            self.webfront.error(test_result)
    
    # template variable
    def formcopy(self, *args):
        if len(args):
            return self.sub['formcopy'](*args)
        ret = ""
        ignore = ["name", "pass", "submit"]
        form = self.webfront.form
        data = self.request.data
        for fieldkey in form.keys():
            if fieldkey in ignore:
                continue
            for fieldvalue in form.getlist(fieldkey):
                ret += self.formcopy(fieldkey, fieldvalue)
        return ret
    

class DeletePage(Template):
    def __init__(self, webfront):
        super(DeletePage, self).__init__(webfront, "delete")
    
    def initialize(self):
        if self.webfront.mode == "post":
            tools.run_callback("pathinfo",
                {"request": self.webfront.request},
                defaultfunc=blosxom_process_path_info)
    
    # template variables
    def delete_type(self):
        form = self.webfront.form
        if form.has_key("delete_type"):
            return form.getfirst("delete_type")
        elif os.path.isfile(self.webfront.pathdir):
            return "file"
        elif form.has_key("file") or form.has_key("directory"):
            return "multiple"
        elif os.path.isdir(self.webfront.pathdir):
            return "directory"
        return "unknown"
    
    def delete_target(self):
        form = self.webfront.form
        if form.has_key("delete_target"):
            return form.getfirst("delete_target")
        if self.delete_type() == "multiple":
            return form.getlist("file") + form.getlist("directory")
        if self.delete_type() == "file" or self.delete_type() == "directory":
            return self.webfront.pathdir
        return "unknown"
    
    def delete_multiple_target(self):
        form = self.webfront.form
        ret = ""
        for self.file in form.getlist('file'):
            ret += self.sub['delete_multiple_target']()
        return ret
    
    def display_form_directory(self):
        pathdir = self.webfront.pathdir
        if self.delete_type() == "directory" and \
           len(os.listdir(pathdir)):
            return True
        if self.delete_type() == "multiple":
            for f in self.webfront.form.getlist('file'):
                if os.path.isdir(os.path.join(pathdir, f)) and \
                   len(os.listdir(pathdir)):
                    return True
        return False
    
    def directory_list(self, selectname):
        webfront = self.webfront
        data = webfront.data
        form = webfront.form
        dirs = ""
        deldir = []
        if self.delete_type() == "directory":
            deldir.append(webfront.http['PATH_INFO'].strip(os.sep))
        if self.delete_type() == "multiple":
            for f in form.getlist('file'):
                if os.path.isdir(webfront.basedir + os.sep +
                   webfront.http['PATH_INFO'] + os.sep + f):
                    deldir.append(webfront.http['PATH_INFO'].strip(os.sep) + os.sep + f)
        def parents(directory, dirlist):
            if not len(dirlist):
                return False
            directory = directory[dirlist[0].rfind(os.sep):]
            for d in dirlist:
                d = d[d.rfind(os.sep)]
                if directory.startswith(d):
                    return True
            return False
        
        filepath = form.getfirst(selectname, os.sep)
        while not os.path.isdir(webfront.basedir + filepath):
            filepath = filepath[:filepath.rfind(os.sep)+1]
        directories = webfront_directory_scan(webfront.basedir, 
            depthlimit=99)
        # directories = webfront.directoryScan(webfront.basedir, 99)
        tools.getLogger().debug(str(directories))
        dirsort = directories.keys()
        dirsort.sort()
        choice = selected = ''
        for directory in dirsort:
            if directory in deldir or parents(directory, deldir):
                continue
            if directory == ".":
                choice = os.sep
            else:
                choice = os.sep + directory + os.sep
            if choice == filepath:
                selected = "selected"
            else:
                selected = ""
            tools.getLogger().debug(str((choice, selected)))
            dirs += self.select_option(choice, selected)
        return dirs
    
    def deletechecked(self):
        form = self.webfront.form
        if form.has_key('contain_function') and \
           form.getfirst('contain_function').lower() == "delete":
            return "checked"
        return ""
    
    def movechecked(self):
        form = self.webfront.form
        if form.has_key('contain_function') and \
           form.getfirst('contain_function').lower() == "move":
            return "checked"
        return ""
    
    # submit response methods
    def submit_delete(self):
        webfront = self.webfront
        error = webfront.error
        flash = webfront.flash
        form = webfront.form
        http = webfront.http
        pathdir = webfront.pathdir
        if self.validate() or self.authenticate():
            return
        if not os.access(pathdir, os.F_OK):
            target = http['PATH_INFO']
            http['PATH_INFO'] = target[:target.rstrip(os.sep).rfind(os.sep)+1]
            webfront.entrypage()
            return
        if not os.access(pathdir, os.F_OK):
            return
        self.cantdelete = []
        self.deleted = []
        deletetype = self.delete_type()
        # process the files/directories
        if deletetype == "file":
            self.deleteFile(pathdir)
        elif deletetype == "directory":
            self.deleteDirectory(pathdir)
        elif deletetype == "multiple":
            for f in form.getlist('file') + form.getlist('directory'):
                path = os.path.join(pathdir, f)
                if os.path.isfile(path):
                    self.deleteFile(path)
                elif os.path.isdir(path):
                    self.deleteDirectory(path)
        # user readable print out of success and failure
        if len(self.deleted):
            if deletetype == "file":
                flash("The file, %s, was successfully deleted." % pathdir)
            elif deletetype == "directory":
                if form.has_key('contain_function'):
                    if form.getfirst('contain_function') == "delete":
                        flash(
                        """The directory, %s, and all of its contents were 
                        successfully deleted.""" % pathdir)
                    elif form.getfirst('contain_function') == "move":
                        flash(
                        """The directory, %s, was deleted and its contents
                        were moved to %s.""" % 
                            (pathdir, form.getfirst('delete_filepath')))
                else:
                    flash("The directory, %s, was successfully deleted." % pathdir)
            elif deletetype == "multiple":
                files = self.deleted
                flash("""The following files and/or directories were 
                successfully deleted: %s.""" % str(files).strip('[]'))
        if len(self.cantdelete):
            if deletetype == "file":
                error("""The file, %s, could not be deleted for the reason: 
                %s.""" % (pathdir, str(self.cantdelete[0][1])))
            elif deletetype == "directory":
                error("""The directory, %s, could not be deleted for the
                reason: %s.""" % (pathdir, str(self.cantdelete[0][1])))
            elif deletetype == "multiple":
                isfiledir = lambda src: os.path.isfile(src) and "file" or "directory"
                reasonlist = [" %s, %s, %s; " % 
                    (src, isfiledir(src), str(reason)) 
                    for src, reason in self.cantdelete]
                error("""The following files and directories could not be
                deleted for their accompanied reasons: %s.""" %
                    str(reasonlist).strip('[];'))
            return
        webfront.entrypage()
    
    def validate(self):
        form = self.webfront.form
        error = self.webfront.error
        # check for directory option if needed
        if self.display_form_directory() and \
           not (form.has_key('contain_function') or \
           form.getfirst('contain_function', "").lower() in ["delete", "move"]):
            error(
            """
            Make sure to select what you want to do with the files and
            directories contained within the directory you are deleting.
            """)
            return True
        # check for password
        if not form.has_key('delete_password'):
            error("Make sure to give your password.")
            return True
    
    def authenticate(self):
        form = self.webfront.form
        conf = self.webfront.conf
        error = self.webfront.error
        user = self.webfront.session['user']
        correctPassword = conf['webfront_users'][user]
        if form.getfirst('delete_password', "").strip() != correctPassword:
            error(
            """
            The password you gave does not match with the password for your
            account, %(user)s.
            """ % {'user': user})
            return True
        return False
    
    def deleteFile(self, src):
        try:
            os.remove(src)
        except StandardError, value:
            self.cantdelete.append(tuple(src, value))
        else:
            self.deleted.append(src)
    
    def deleteDirectory(self, src):
        form = self.webfront.form
        try:
            if len(os.listdir(src)):
                # contains files, find out what to do with the contents
                contain_function = form.getfirst('contain_function')
                if contain_function == "delete":
                    # get rid of everything
                    shutil.rmtree(src)
                elif contain_function == "move":
                    contents = os.listdir(src)
                    dst = self.webfront.basedir + form.getfirst('delete_filepath')
                    replacing = self.webfront.replacecheck([dst + x for x in contents], self.webfront.mode == "post")
                    if replacing == True or replacing == "not replacing":
                        for f in contents:
                            shutil.move(os.path.join(src, f), os.path.join(dst, f))
                    elif isinstance(replacing, str):
                        raise StandardError, \
                        """The files within %(src)s cannot be moved as there
                        are files that share there names in the 
                        destination""" % {'src': src}
                    os.rmdir(src)
                else:
                    raise Exception, """Deleting of the directory %s was 
                    handled incorrectly.""" % src
            else:
                os.rmdir(src)
        except StandardError, value:
            self.cantdelete.append((src, value))
        else:
            self.deleted.append(src)
    


# Begin revised classes
def getTemplate(name):
    if templatePath() not in sys.path:
        sys.path.append(templatePath())
    Cheetah.ImportHooks.install()
    try:
        tmplclass = Cheetah.Template.Template.compile(
            file = os.path.join(templatePath(), name),
            moduleName=name.split(os.extsep)[0],
            className=name.split(os.extsep)[0],
            cacheCompilationResults=True,
            useCache=True,
            cacheModuleFilesForTracebacks=True,
            cacheDirForModuleFiles=temporaryPath())
    finally:
        Cheetah.ImportHooks.uninstall()
    return tmplclass

class WebHandle(object):
    """
    Base class for all webfront page handles.
    
    Assign a string value with the path of the template for that handler in
    relation to the template directory, to a variable named 'tmplname'.  
    Otherwise it will default to the base template, which will display, 
    practically no useful info.
    """
    def __call__(self, webfront):
        if not hasattr(self, "webfront"):
            self.webfront = webfront
        self.webfront.template = self
        if hasattr(self, "start"):
            self.start()
        if self.webfront.result[0]:
            for key in self.webfront.form.keys():
                if key.startswith('submit') and \
                   hasattr(self, key) and callable(eval("self." + key)):
                    eval("self." + key)()
        if hasattr(self, "end"):
            self.end()
    
    def submit_cancel(self):
        self.webfront.result = False
        self.webfront.entrypage()
    
    def render(self, tmplfunction=None, *args, **kwargs):
        if len(args) > 0 or len(kwargs) > 0:
            return
        if not hasattr(self, "tmplname"):
            self.tmplname = "base.tmpl"
        tmpl = getTemplate(self.tmplname)(searchList = [self.webfront])
        response = self.webfront.request.getResponse()
        response.addHeader('Content-Type', 'text/html')
        if hasattr(self, 'start'):
            self.start()
        if tmplfunction == None:
            response.write(str(tmpl))
        elif tmplfunction not in dir(tmpl):
            pass
        else:
            response.write(eval('tmpl.' + tmplfunction + '()'))
        if hasattr(self, 'end'):
            self.end()
    

class ConfigHandle(WebHandle):
    tmplname = "config.tmpl"
    def submit_save(self):
        """Saves the changes made to the form of the config template to
        config.py and the current config."""
        from configedit import ConfigEditor, ConfigVariable
        # self.webfront.error('Called')
        ce = ConfigEditor()
        form = self.webfront.form
        for key in filter(lambda key: key.startswith('variable_name_'), 
                          form.keys()):
            name = form.getfirst(key)
            keyname = key[len('variable_name_'):]
            owner = form.getfirst('variable_owner_' + keyname)
            vlist = form.getlist('variable_value_' + keyname)
            vtype = form.getfirst('variable_type_' + keyname)
            enabled = eval(form.getfirst('variable_enabled_' + keyname, 
                                         "False"))
            value = None
            if vtype == "list":
                value = []
                for v in vlist:
                    try:
                        v = eval(v)
                    except:
                        pass
                    value.append(v)
            elif vtype == "dict":
                value = {}
                for v in vlist:
                    k = v[:v.find(':')]
                    v = v[v.find(':')+1:].strip()
                    try:
                        k = eval(k)
                    except:
                        pass
                    try:
                        v = eval(v)
                    except:
                        pass
                    value[k] = v
            elif vtype == "single" or vtype == "unknown":
                try:
                    value = eval(vlist[0])
                except:
                    value = vlist[0]
            args = {'name': keyname, 'value': value}
            tools.run_callback('webfront_config_save', args)
            value = args['value']
            if ce.hasVariable(name):
                var = ce.getVariable(name)
                var.value = value
                var.enabled = enabled
            else:
                section = ce.getSection(owner)
                section.addVariable(ConfigVariable(name, enabled, value))
        ce.save()
    
    def change_current_config(self):
        """
        Not Implemented.
        
        Need to find or see if it is possible to update the config variables
        for future requests if by PyBlosxom is being run as a long term 
        process.
        """
        pass
    

class EditHandle(WebHandle):
    tmplname = "edit.tmpl"
    def start(self):
        if self.webfront.mode != "post":
            return
        
        tools.run_callback("pathinfo",
            {"request": self.webfront.request},
            defaultfunc=blosxom_process_path_info)
        
        http = self.webfront.http
        data = self.webfront.data
        # special handling for drafts.
        if os.path.splitext(http['PATH_INFO'])[1] == ".draft" and \
           'draft' in data['extensions']:
            # if viewing a draft, remove all other extensions
            self.allextensions = data['extensions']
            data['extensions'] = {'draft': data['extensions']['draft']}
        elif 'draft' in data['extensions']:
            # don't look for drafts
            self.draftext = data['extensions']['draft']
            del data['extensions']['draft']
    
    def end(self):
        if self.webfront.mode != "post":
            return
        data = self.webfront.data
        http = self.webfront.http
        if hasattr(self, 'allextensions'):
            data['extensions'] = self.allextensions
            del self.allextensions
        elif hasattr(self, 'draftext'):
            data['extensions']['draft'] = self.draftext
            del self.draftext
    
    def submit_save(self):
        self.method = "save"
        if self.validate() or self.replacing():
            return
        form = self.webfront.form
        if self.webfront.mode == "post":
            metadata = self.getMetadata()
            if 'webfront:extension' in metadata:
                del metadata['webfront:extension']
            save_changes(
                self.filepath, 
                form.getfirst("edit_title", ""), 
                form.getfirst("edit_body", ""),
                metadata
            )
            
            # save the new date
            datestr = form.getfirst("edit_date", "")
            if datestr != "":
                try:
                    timetuple = time.strptime(datestr)
                    timestamp = time.mktime(timetuple)
                    stat = os.stat(self.filepath)
                    atime = stat[7]
                    os.utime(self.filepath, (atime, timestamp))
                except Exception, e:
                    self.webfront.error(str(e))
                    self.removePrevious()
                    return
        else:
            f = open(self.filepath, 'w')
            f.write(form.getfirst('edit_body', ""))
            f.close()
        self.removePrevious()
        self.returnToEntry()
    
    def submit_draft(self):
        self.method = "draft"
        if self.validate() or self.replacing():
            return
        form = self.webfront.form
        draftpath = self.filepath[:self.filepath.rfind(os.extsep)+1] + self.fileext
        save_draft(
            draftpath, 
            form.getfirst('edit_title', ""), 
            form.getfirst('edit_body', ""),
            self.getMetadata())
        # self.removePrevious()
        self.returnToEntry()
    
    def submit_preview(self):
        self.method = "preview"
        if self.validate() or self.replacing():
            return
        form = self.webfront.form
        draftpath = self.filepath[:self.filepath.rfind(os.extsep)+1] + self.fileext
        save_draft(draftpath, form.getfirst('edit_title', ""), form.getfirst('edit_body', ""))
        # self.removePrevious()
        self.webfront.previewpage()
    
    # form response associated methods
    def getMetadata(self):
        metadata = self.webfront.form.getfirst('edit_meta', "")
        metadata = metadata.split('\r')
        metadata = map(lambda a: a.split('\n'), metadata)
        metadata = reduce(lambda a, b: a + b, metadata)
        metadata = [(s[:s.find(' ')], s[s.find(' ')+1:]) \
                    for s in metadata if len(s)]
        return dict(metadata)
    
    def validate(self):
        # Validation of fields
        form = self.webfront.form
        error = self.webfront.error
        if not form.has_key('edit_filename') or form.getfirst('edit_filename') == "":
            error("Your post must have a file name.")
            return True
        filename = form.getfirst('edit_filename', "")
        if not filename.replace(os.extsep, '').replace('_', '').replace('-', '').isalnum():
            error("File name must only contain letters of the alphabet, numerals, periods, underscores, and dashes.")
            return True
        return False
    
    def replacing(self):
        form = self.webfront.form
        http = self.webfront.http
        # does the path_info and file name and path not match or is 
        # it new?  Additionally, does a file already exist there?
        self.filepath = self.webfront.basedir + form.getfirst("edit_filepath", os.sep) + form.getfirst("edit_filename")
        filename = self.filepath[self.filepath.rfind(os.sep)+1:self.filepath.rfind(os.extsep)]
        self.filename = filename
        fileext = self.filepath[self.filepath.rfind(os.extsep)+1:]
        self.fileext = fileext
        if self.method == "draft" or self.method == "preview":
            self.filepath = self.filepath[:self.filepath.rfind(os.extsep)+1] + "draft"
        special = eval(form.getfirst("edit_special", "[]"))
        replacing = self.webfront.replacecheck(self.filepath, ignoreext=self.webfront.mode == "post")
        if replacing == False:
            # they didn't want to replace the file, let them change the 
            # file name
            self.webfront.flash("Make sure to name your file something else as %s is already taken." % (form.getfirst("edit_filepath", os.sep) + form.getfirst("edit_filename")))
            self.webfront.result = False
            self.webfront.renderpage("edit")
            return True
        elif not replacing == True and replacing != 'not replacing' and \
            ('newentry' in special or self.filepath != self.webfront.basedir + http['PATH_INFO']) and \
            not self.filepath[:self.filepath.rfind(os.sep)+1] + replacing == self.webfront.pathdir:
            # the user is attempting to replace a file, we should confirm this decision
            replacing = form.getfirst("edit_filepath", os.sep) + replacing
            oldfile = 'newentry' in special and "a new file" or http['PATH_INFO']
            self.webfront.replacepage(replacing, oldfile)
            return True
        else:
            return False
    
    def removePrevious(self):
        http = self.webfront.http
        # we may need to remove any files with the same name or the old file 
        # if the name changed.
        if self.filepath != self.webfront.pathdir and \
           os.path.isfile(self.webfront.pathdir):
            # the name or extension or both have changed and the old file 
            # exists
            os.remove(self.webfront.pathdir)
        oldpath = http['PATH_INFO'][:http['PATH_INFO'].rfind(os.sep) + 1]
        newpath = self.webfront.getfield('self.filepath', os.sep)
        if oldpath != newpath:
            # we've changed directory so remove any file with the same name
            files = os.listdir(self.filepath[:self.filepath.rfind(os.sep)])
            for f in files:
                if f.startswith(self.filename) and \
                   not f.endswith(self.method == "save" and self.fileext or "draft"):
                    os.remove(self.filepath[:self.filepath.rfind(os.sep)+1] + f)
    
    def returnToEntry(self):
        pyhttp = self.webfront.http
        pyhttp['PATH_INFO'] = pyhttp['PATH_INFO'][:pyhttp['PATH_INFO'].rfind(os.sep)+1]
        if self.method == "draft":
            self.filepath = self.filepath[:self.filepath.rfind(os.extsep)] + ".draft"
        self.webfront.flash("Changes to %s were saved." % self.filepath)
        self.webfront.entrypage(runpathinfo=True)
    

class EntryHandle(WebHandle):
    tmplname = "entry.tmpl"

