# Copyright (c) 2007 Michael "Z" Goddard
# This software is distributed under the MIT license.

"""
Base template code used by the webfront's pages.

In the process of being deprecated.  Templates to be written in Cheetah.
"""

import os

from Pyblosxom.pyblosxom import VERSION as PYBL_VERSION
from Pyblosxom.pyblosxom import Request
from Pyblosxom import tools

from webfront import *
from fileops import *

# pyblosxom 1.5 changes the api for tools.parse
parse_template = lambda *a: ""
if PYBL_VERSION.startswith("1.5"):
    # drop the encoding variable
    parse_template = lambda request, encoding, vardict, template: \
        tools.parse(request, vardict, template)
else:
    parse_template = tools.parse

class Template(dict):
    def getBaseSubs(cls):
        if not hasattr(cls, "_basesubs"):
            cls._basesubs = None
        return cls._basesubs
    
    def setBaseSubs(cls, value):
        cls._basesubs = value
    
    getBaseSubs = classmethod(getBaseSubs)
    setBaseSubs = classmethod(setBaseSubs)
    basesubs = property(getBaseSubs, setBaseSubs)
    def __init__(self, webfront, filepath):
        self.webfront = webfront
        self.request = webfront.request
        self._filepath = filepath
        self._filename = filepath[filepath.rfind(os.sep)+1:]
        if not os.path.isfile(filepath):
            self._filepath = os.path.join(templatePath(), filepath)
        if not os.path.isfile(self._filepath):
            raise TemplateException, "Unable to find template file '%s'" % filepath
        self.core = None
        self.sub = None
    
    def __getitem__(self, item):
        if hasattr(self, item):
            if callable(eval("self." + item)) and not isinstance(eval("self." + item), (SubTemplate)):
                return DropRequest(eval("self." + item), "self." + item)
            if isinstance(eval("self." + item), (str, unicode)):
                return eval("self." + item)
        if self.sub.has_key(item):
            if isinstance(self.sub[item], SubTemplate):
                return self.sub[item]
        # if isinstance(item, unicode):
        #     item = str(item)
        if self.request.data.has_key(item):
            return self.request.data[item]
        if self.request.conf.has_key(item):
            return self.request.conf[item]
        if self._rendered.has_key(item):
            return self._rendered[item]
        return super(Template, self).__getitem__(item)
    
    def __getattr__(self, name):
        if self.sub == None:
            self._loadTemplate()
        if self.sub.has_key(name):
            if isinstance(self.sub[name], SubTemplate):
                return self.sub[name]
        raise AttributeError, name
    
    def __repr__(self):
        c = self.__class__.__name__
        return "<%s filepath='%s'>" % (c, self._filepath)
    
    def __call__(self, webfront):
        if not hasattr(self, "webfront"):
            self.webfront = webfront
        self.webfront.template = self
        if self.webfront.result[0]:
            for key in self.webfront.form.keys():
                if key.startswith('submit') and \
                   hasattr(self, key) and callable(eval("self." + key)):
                    eval("self." + key)()
        if hasattr(self, "initialize") and callable(self.initialize):
            self.initialize()
    
    def _loadTemplate(self):
        if self.core == None:
            filetext = open(self._filepath, 'r').read()
            if isinstance(Template.basesubs, dict):
                self.sub = dict(Template.basesubs)
            else:
                self.sub = {}
                basetext = open(os.path.join(templatePath(), "base")).read()
                pos = basetext.find("<template")
                while not pos == -1:
                    name = basetext[basetext.find('name="', pos)+6:basetext.find('"', basetext.find('name="', pos)+6)]
                    # if basetext.find('repeatable', pos, basetext.find('>', pos)):
                    #     self.repeats[name]
                    self.sub[name] = basetext[basetext.find('>', pos)+1:basetext.find('</template>', pos)]
                    if basetext.find('args="', pos, basetext.find('>', pos)) != -1:
                        args = basetext[basetext.find('args="', pos)+6:
                                        basetext.find('"', basetext.find('args="', pos)+6, basetext.find('>', pos))]
                        self.sub[name] = FuncTemplate(self, name, args, self.sub[name])
                    elif filetext.find('cond="', pos, filetext.find('>', pos)) != -1:
                        cond = filetext[filetext.find('cond="', pos)+6:
                                        filetext.find('"', filetext.find('cond="', pos)+6, filetext.find('>', pos))]
                        self.sub[name] = CondTemplate(self, name, cond, self.sub[name])
                    else:
                        self.sub[name] = SubTemplate(self, name, self.sub[name])
                    pos = basetext.find('<template', pos+1)
                Template.basesubs = dict(self.sub)
            self._rendered = {}
            pos = filetext.find("</html>")+7
            self.core = filetext[:pos]
            pos = filetext.find("<template", pos)
            while not pos == -1:
                name = filetext[filetext.find('name="', pos)+6:filetext.find('"', filetext.find('name="', pos)+6)]
                self.sub[name] = filetext[filetext.find('>', pos)+1:filetext.find('</template>', pos)]
                if filetext.find('args="', pos, filetext.find('>', pos)) != -1:
                    args = filetext[filetext.find('args="', pos)+6:
                                    filetext.find('"', filetext.find('args="', pos)+6, filetext.find('>', pos))]
                    self.sub[name] = FuncTemplate(self, name, args, self.sub[name])
                elif filetext.find('cond="', pos, filetext.find('>', pos)) != -1:
                    cond = filetext[filetext.find('cond="', pos)+6:
                                    filetext.find('"', filetext.find('cond="', pos)+6, filetext.find('>', pos))]
                    self.sub[name] = CondTemplate(self, name, cond, self.sub[name])
                else:
                    self.sub[name] = SubTemplate(self, name, self.sub[name])
                pos = filetext.find('<template', pos+1)
    
    def has_key(self, key):
        if hasattr(self, key) and not isinstance(eval("self." + key), (SubTemplate, FuncTemplate)):
            return True
        if self.sub.has_key(key) and \
           isinstance(self.sub[key], SubTemplate):
            return True
        if self.request.data.has_key(key):
            return True
        if self.request.conf.has_key(key):
            return True
        if self._rendered.has_key(key):
            return True
        return super(Template, self).has_key(key)
    
    def reset(self, subname):
        
        self._loadTemplate()
        self._rendered[subname] = ""
    
    def render(self, subname = None, repeatsub=True):
        self._loadTemplate()
        if subname:
            if not subname in self.sub:
                raise SubtemplateMissingException, \
                      "Missing %s template in %s." % (subname, self)
            config = self.request.conf
            # self.update(self.request.data)
            # self.update(config)
            # self.update(self._rendered)
            
            if not self._rendered.has_key(subname) or not repeatsub:
                self._rendered[subname] = ""
            self._rendered[subname] += parse_template(self.request, 
                config["blog_encoding"], self, self.sub[subname].text)
            return
        encoding = self.request.conf['blog_encoding']
        
        response = self.request.getResponse()
        response.addHeader('Content-Type', "text/html")
        response.addHeader('Content-Encoding', encoding)
        
        parsed = parse_template(self.request, encoding, self, self.core)
        
        response.addHeader('Content-Length', str(len(parsed)))
        response.write(parsed)
    
    def pageBar(self):
        data = self.webfront.data
        links = ""
        for link, path in self.webfront.links:
            links += self.linktab_top(path, link)
        return links
    
    def error(self):
        if self._rendered.has_key('error'):
            return self._rendered['error']
        return ""
    
    def flash(self):
        if self._rendered.has_key('flash'):
            return self._rendered['flash']
        return ""
    
    def submit_cancel(self):
        self.webfront.result = False
        self.webfront.entrypage()
    

class DropRequest:
    """
    Small class that wraps a function and drops the Request object if it is 
    the first argument.
    """
    def __init__(self, func, attrname):
        self.func = func
        self.attrname = attrname
    
    def __call__(self, *args, **kwargs):
        if len(args) and \
           isinstance(args[0], Request):
            args = args[1:]
        if len(args) and \
           "im_self" in dir(self.func) and \
           args[0] == self.func.im_self:
            args = args[1:]
        print args
        return self.func(*args, **kwargs)
    
    def __repr__(self):
        c = self.__class__.__name__
        return "<%s for attribute \"%s\">" % (c, self.attrname)
    

class SubTemplate(object):
    """
    Subtemplates are are parts of a template defining the look of optional or
    repeated section.
    """
    def __init__(self, parent, name, text):
        self.parent = parent
        self.name = name
        self.text = text
    
    def __call__(self, request=None, *args):
        return parse_template(self.parent.request, 
            self.parent.request.config["blog_encoding"], 
            self.parent, self.text)
    
    def __repr__(self):
        c = self.__class__.__name__
        return "<%s name=\"%s\">" % (c, self.name)
    
    def __str__(self):
        if isinstance(self.parent, Template):
            return self.parent._rendered[self.name]
        return ""
    

class FuncTemplate(SubTemplate):
    """
    Function Templates are subtemplates that can be called with arguments
    either from the templates or code.
    """
    def __init__(self, parent, name, args, text):
        super(FuncTemplate, self).__init__(parent, name, text)
        self.args = args.split(',')
        
        for index in range(len(self.args)):
            self.args[index] = self.args[index].strip()
    
    def __call__(self, *args, **kwargs):
        if len(args) and \
           isinstance(args[0], Request):
            args = args[1:]
        if len(args) and \
           isinstance(args[0], Template):
            args = args[1:]
        
        parse = lambda template: parse_template(
            self.parent.request, 
            self.parent.request.config['blog_encoding'],
            self.parent,
            template)
        for arg in self.args:
            if arg in kwargs:
                self.parent[arg] = parse(kwargs[arg])
        argsused = 0
        for arg in self.args:
            if arg not in kwargs:
                self.parent[arg] = parse(args[argsused])
                argsused += 1
        return parse(self.text)
    

class CondTemplate(SubTemplate):
    def __init__(self, parent, name, cond, text):
        super(CondTemplate, self).__init__(parent, name, text)
        self.cond = cond
    
    def __call__(self, request=None):
        parse = lambda template: parse_template(
            self.parent.request, 
            self.parent.request.config['blog_encoding'],
            self.parent,
            template)
        if eval(parse(self.cond)):
            return parse(self.text)
        return ""
    

class TemplateException(Exception):
    pass

class MalformedTemplateException(TemplateException):
    pass

class SubtemplateMissingException(TemplateException):
    pass

